import 'package:dartdoku/domains/sudoku/SudokuProblem.dart';

import 'api_dartdoku.dart';
import 'package:dartdoku/dartdoku.dart' as dartdoku;

/// This type initializes an application.
///
/// Override methods in this class to set up routes and initialize services like
/// database connections. See http://aqueduct.io/docs/http/channel/.
class ApiDartdokuChannel extends ApplicationChannel {
  /// Initialize services in this method.
  ///
  /// Implement this method to initialize services, read values from [options]
  /// and any other initialization required before constructing [entryPoint].
  ///
  /// This method is invoked prior to [entryPoint] being accessed.
  @override
  Future prepare() async {
    logger.onRecord.listen(
        (rec) => print("$rec ${rec.error ?? ""} ${rec.stackTrace ?? ""}"));
  }

  /// Construct the request channel.
  ///
  /// Return an instance of some [Controller] that will be the initial receiver
  /// of all [Request]s.
  ///
  /// This method is invoked after [prepare].
  @override
  Controller get entryPoint {
    final router = Router();

    // Prefer to use `link` instead of `linkFunction`.
    // See: https://aqueduct.io/docs/http/request_controller/
    var base = '/api/v1';
    router.route("$base/hello").link(() => HelloController());

    router
        .route("$base/hints/:hints[/count/:count]")
        .link(() => SudokuController());
    router
        .route("$base/count/:count[/hints/:hints]")
        .link(() => SudokuController());
    router.route("$base").link(() => SudokuController());

    return router;
  }
}

class SudokuController extends ResourceController {
  Future<List<dynamic>> getBoards(int hints, int count) async {
    if (hints >= 17 && hints <= 81) {
      var problems = await dartdoku.getProblems(hints, count);
      var boards = [];
      for (SudokuProblem problem in problems) {
        boards.add({
          "initial board":
              SudokuProblem.boardToString(problem.getInitialState()),
          "final board": SudokuProblem.boardToString(problem.getFinalState())
        });
      }
      return boards;
    } else {
      throw Error();
    }
  }

  @Operation.get('hints', 'count')
  Future<Response> getNewGameHintsCount(@Bind.path('hints') int hints,
      {@Bind.path('count') int count = 1}) async {
    if (hints < 17 || hints > 81) {
      return Response.badRequest(body: 'hints must be within [17,81]');
    } else if (count < 1) {
      return Response.badRequest(body: 'count must be positive');
    } else {
      final boards = await getBoards(hints, count);
      return Response.ok({'boards': boards});
    }
  }

  @Operation.get('hints')
  Future<Response> getSingleGame(@Bind.path('hints') int hints) async {
    if (hints < 17 || hints > 81) {
      return Response.badRequest(body: 'hints must be within [17,81]');
    } else {
      final boards = await getBoards(hints, 1);
      return Response.ok({'boards': boards});
    }
  }

  @Operation.get('count')
  Future<Response> getMultiGame(@Bind.path('count') int count) async {
    if (count < 1) {
      return Response.badRequest(body: 'count must be positive');
    } else {
      final boards = await getBoards(17, count);
      return Response.ok({'boards': boards});
    }
  }

  @Operation.get()
  Future<Response> getSingleGameNoExtraHints() async {
    final boards = await getBoards(17, 1);
    return Response.ok({'boards': boards});
  }
}

class HelloController extends ResourceController {
  @Operation.get()
  Future<Response> hello() async {
    final now = DateTime.now().toLocal();
    final year = now.year;
    final month = getMonth(now.month);
    final day = now.day.toString().padLeft(2, '0');
    final weekday = getWeekday(now.weekday);
    final ending = now.hour > 11 ? 'pm' : 'am';
    final hour = getHour(now.hour).toString().padLeft(2, '0');
    final minute = now.minute.toString().padLeft(2, '0');
    final second = now.second.toString().padLeft(2, '0');
    return Response.ok(
        'Hello I am api_dartdoku and it is currently $weekday, $day $month $year at $hour:$minute:$second $ending.');
  }
}

String getMonth(int month) {
  final Map<int, String> monthMap = {
    1: 'January',
    2: 'February',
    3: 'March',
    4: 'April',
    5: 'May',
    6: 'June',
    7: 'July',
    8: 'August',
    9: 'September',
    10: 'October',
    11: 'November',
    12: 'December'
  };
  return monthMap[month] ?? 'month';
}

String getWeekday(int weekday) {
  final Map<int, String> weekMap = {
    0: 'Sunday',
    1: 'Monday',
    2: 'Tuesday',
    3: 'Wednesday',
    4: 'Thursday',
    5: 'Friday',
    6: 'Saturday'
  };
  return weekMap[weekday] ?? 'weekday';
}

int getHour(int hour) {
  if (hour > 12) {
    return hour - 12;
  } else if (hour == 0) {
    return 12;
  } else {
    return hour;
  }
}
